import itertools
import typing as t
from datetime import datetime

import pytest

from src.scraper.station import Station
from src.scraper.train import Train


@pytest.mark.parametrize(
    "kind, station_code",
    itertools.product(
        ("partenze", "arrivi"),
        [
            "S01700",
            "S08409",
            "S09218",
            "S01608",
            "N00001",
            "N00005",
        ],
    ),
)
def test_fetch(kind, station_code):
    station: Station = Station.by_code(station_code)
    trains: t.List[Train] = (
        station.departures() if kind == "partenze" else station.arrivals()
    )
    for train in trains:
        train.fetch()
        if (
            not train.departed
            and not train._phantom
            and not train._trenord_phantom
            and not train.cancelled
        ):
            assert not train.arrived()


def test_unfetched_repr_1():
    milan: Station = Station.by_code("S01700")
    train: Train = Train(10911, milan, datetime.now().date())
    assert repr(train) == "Treno [?] ??? 10911 : Milano Centrale [S01700@1] -> ???"


def test_unfetched_repr_2():
    train: Train = Train._from_station_departures_arrivals(
        {
            "numeroTreno": 10911,
            "codOrigine": "S01700",
            "categoriaDescrizione": "REG",
            "dataPartenzaTreno": 1678662000000,
            "codiceCliente": 1,
            "nonPartito": False,
            "provvedimento": 0,
            "compImgCambiNumerazione": "",
        }
    )
    assert repr(train) == "Treno [D] REG 10911 : Milano Centrale [S01700@1] -> ???"


def test_hash():
    milan: Station = Station.by_code("S01700")
    trains: list[Train] = milan.departures()
    if not trains:
        return
    assert hash(trains[0]) is not None
